@@ -28,10 +28,15 @@ using System;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Serialization;
 using System.Reflection;
+using System.Xml;
+using System.Text;
+using System.Linq;
+using MonoDevelop.Projects.Policies;
 
 namespace MonoDevelop.CSharp.Formatting
 {
-	public enum BraceStyle {
+	public enum BraceStyle
+	{
 		DoNotChange,
 		EndOfLine,
 		EndOfLineWithoutSpace,
@@ -40,24 +45,45 @@ namespace MonoDevelop.CSharp.Formatting
 		NextLineShifted2
 	}
 	
-	public enum BraceForcement {
+	public enum BraceForcement
+	{
 		DoNotChange,
 		RemoveBraces,
 		AddBraces
 	}
 	
-	public enum ArrayInitializerPlacement {
+	public enum ArrayInitializerPlacement
+	{
 		AlwaysNewLine,
 		AlwaysSameLine
 	}
 	
+	public enum PropertyFormatting
+	{
+		AllowOneLine,
+		ForceOneLine,
+		ForceNewLine
+	}
+	
+	[PolicyType ("C# formatting")]
 	public class CSharpFormattingPolicy : IEquatable<CSharpFormattingPolicy>
 	{
+		public string Name {
+			get;
+			set;
+		}
+		
+		public bool IsBuiltIn {
+			get;
+			set;
+		}
+		
 		public CSharpFormattingPolicy Clone ()
 		{
-			return (CSharpFormattingPolicy) MemberwiseClone ();
+			return (CSharpFormattingPolicy)MemberwiseClone ();
 		}
 		
+		
 		#region Indentation
 		[ItemProperty]
 		public bool IndentNamespaceBody { // tested
@@ -130,6 +156,25 @@ namespace MonoDevelop.CSharp.Formatting
 			get;
 			set;
 		}
+		
+		[ItemProperty]
+		public bool AlignEmbeddedUsingStatements { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AlignEmbeddedIfStatements { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public PropertyFormatting PropertyFormatting { // tested
+			get;
+			set;
+		}
+		
 		#endregion
 		
 		#region Braces
@@ -338,6 +383,38 @@ namespace MonoDevelop.CSharp.Formatting
 		#endregion
 		
 		#region Spaces
+		// Methods
+		[ItemProperty]
+		public bool BeforeMethodDeclarationParentheses { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool BetweenEmptyMethodDeclarationParentheses {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool BeforeMethodDeclarationParameterComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AfterMethodDeclarationParameterComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool WithinMethodDeclarationParentheses { // tested
+			get;
+			set;
+		}
+		
+		// Method calls
 		[ItemProperty]
 		public bool BeforeMethodCallParentheses { // tested
 			get;
@@ -345,11 +422,59 @@ namespace MonoDevelop.CSharp.Formatting
 		}
 		
 		[ItemProperty]
-		public bool BeforeMethodDeclarationParentheses { // tested
+		public bool BetweenEmptyMethodCallParentheses { // tested
 			get;
 			set;
 		}
 		
+		[ItemProperty]
+		public bool BeforeMethodCallParameterComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AfterMethodCallParameterComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool WithinMethodCallParentheses { // tested
+			get;
+			set;
+		}
+		
+		// fields
+		
+		[ItemProperty]
+		public bool BeforeFieldDeclarationComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AfterFieldDeclarationComma { // tested
+			get;
+			set;
+		}
+		
+		// local variables
+		
+		[ItemProperty]
+		public bool BeforeLocalVariableDeclarationComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AfterLocalVariableDeclarationComma { // tested
+			get;
+			set;
+		}
+		
+		// constructors
+		
 		[ItemProperty]
 		public bool BeforeConstructorDeclarationParentheses { // tested
 			get;
@@ -357,11 +482,87 @@ namespace MonoDevelop.CSharp.Formatting
 		}
 		
 		[ItemProperty]
-		public bool BeforeDelegateDeclarationParentheses { // tested
+		public bool BetweenEmptyConstructorDeclarationParentheses { // tested
 			get;
 			set;
 		}
 		
+		[ItemProperty]
+		public bool BeforeConstructorDeclarationParameterComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AfterConstructorDeclarationParameterComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool WithinConstructorDeclarationParentheses { // tested
+			get;
+			set;
+		}
+		
+		// indexer
+		[ItemProperty]
+		public bool BeforeIndexerDeclarationBracket { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool WithinIndexerDeclarationBracket { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool BeforeIndexerDeclarationParameterComma {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AfterIndexerDeclarationParameterComma {
+			get;
+			set;
+		}
+		
+		// delegates
+		
+		[ItemProperty]
+		public bool BeforeDelegateDeclarationParentheses {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool BetweenEmptyDelegateDeclarationParentheses {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool BeforeDelegateDeclarationParameterComma {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AfterDelegateDeclarationParameterComma {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool WithinDelegateDeclarationParentheses {
+			get;
+			set;
+		}
+		
+		
 		[ItemProperty]
 		public bool NewParentheses { // tested
 			get;
@@ -464,23 +665,18 @@ namespace MonoDevelop.CSharp.Formatting
 			set;
 		}
 		
+		[ItemProperty]
+		public bool AroundNullCoalescingOperator {
+			get;
+			set;
+		}
+		
 		[ItemProperty]
 		public bool WithinParentheses { // tested
 			get;
 			set;
 		}
 		
-		[ItemProperty]
-		public bool WithinMethodCallParentheses { // tested
-			get;
-			set;
-		}
-		
-		[ItemProperty]
-		public bool WithinMethodDeclarationParentheses { // tested
-			get;
-			set;
-		}
 		
 		[ItemProperty]
 		public bool WithinIfParentheses { // tested
@@ -542,12 +738,48 @@ namespace MonoDevelop.CSharp.Formatting
 			set;
 		}
 		
+		[ItemProperty]
+		public bool BeforeSizeOfParentheses { // tested
+			get;
+			set;
+		}
+		
 		[ItemProperty]
 		public bool WithinTypeOfParentheses { // tested
 			get;
 			set;
 		}
 		
+		[ItemProperty]
+		public bool WithinNewParentheses { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool BetweenEmptyNewParentheses { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool BeforeNewParameterComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool AfterNewParameterComma { // tested
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public bool BeforeTypeOfParentheses { // tested
+			get;
+			set;
+		}
+		
 		[ItemProperty]
 		public bool WithinCheckedExpressionParantheses { // tested
 			get;
@@ -578,26 +810,37 @@ namespace MonoDevelop.CSharp.Formatting
 			set;
 		}
 		
+		// brackets
 		[ItemProperty]
 		public bool SpacesWithinBrackets { // tested
 			get;
 			set;
 		}
+		[ItemProperty]
+		public bool SpacesBeforeBrackets { // tested
+			get;
+			set;
+		}
+		[ItemProperty]
+		public bool BeforeBracketComma { // tested
+			get;
+			set;
+		}
+		[ItemProperty]
+		public bool AfterBracketComma { // tested
+			get;
+			set;
+		}
+		
 		
 		[ItemProperty]
-		public bool SpacesAfterComma { // tested
+		public bool SpacesBeforeForSemicolon { // tested
 			get;
 			set;
 		}
 		
 		[ItemProperty]
-		public bool SpacesBeforeComma { // tested
-			get;
-			set;
-		}
-		
-		[ItemProperty]
-		public bool SpacesAfterSemicolon { // tested
+		public bool SpacesAfterForSemicolon { // tested
 			get;
 			set;
 		}
@@ -607,10 +850,61 @@ namespace MonoDevelop.CSharp.Formatting
 			get;
 			set;
 		}
+		
+		[ItemProperty]
+		public bool SpacesBeforeArrayDeclarationBrackets { // tested
+			get;
+			set;
+		}
+		#endregion
+		
+		#region Blank Lines
+		[ItemProperty]
+		public int BlankLinesBeforeUsings {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public int BlankLinesAfterUsings {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public int BlankLinesBeforeFirstDeclaration {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public int BlankLinesBetweenTypes {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public int BlankLinesBetweenFields {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public int BlankLinesBetweenEventFields {
+			get;
+			set;
+		}
+		
+		[ItemProperty]
+		public int BlankLinesBetweenMembers {
+			get;
+			set;
+		}
 		#endregion
 		
 		public CSharpFormattingPolicy ()
 		{
+			
 			IndentNamespaceBody = true;
 			IndentClassBody = IndentInterfaceBody = IndentStructBody = IndentEnumBody = true;
 			IndentMethodBody = IndentPropertyBody = IndentEventBody = true;
@@ -640,8 +934,15 @@ namespace MonoDevelop.CSharp.Formatting
 			BeforeMethodDeclarationParentheses = true;
 			BeforeConstructorDeclarationParentheses = true;
 			BeforeDelegateDeclarationParentheses = true;
-
+			AfterMethodCallParameterComma = true;
+			AfterConstructorDeclarationParameterComma = true;
+			
 			NewParentheses = true;
+			WithinNewParentheses = false;
+			BetweenEmptyNewParentheses = false;
+			BeforeNewParameterComma = false;
+			AfterNewParameterComma = true;
+			
 			IfParentheses = true;
 			WhileParentheses = true;
 			ForParentheses = true;
@@ -658,6 +959,7 @@ namespace MonoDevelop.CSharp.Formatting
 			AroundAdditiveOperatorParentheses = true;
 			AroundMultiplicativeOperatorParentheses = true;
 			AroundShiftOperatorParentheses = true;
+			AroundNullCoalescingOperator = true;
 			WithinParentheses = false;
 			WithinMethodCallParentheses = false;
 			WithinMethodDeclarationParentheses = false;
@@ -679,20 +981,115 @@ namespace MonoDevelop.CSharp.Formatting
 			ConditionalOperatorAfterSeparatorSpace = true;
 
 			SpacesWithinBrackets = false;
-			SpacesAfterComma = true;
-			SpacesBeforeComma = false;
-			SpacesAfterSemicolon = true;
+			SpacesBeforeBrackets = true;
+			BeforeBracketComma = false;
+			AfterBracketComma = true;
+					
+			SpacesBeforeForSemicolon = false;
+			SpacesAfterForSemicolon = true;
 			SpacesAfterTypecast = false;
+			
+			AlignEmbeddedIfStatements = true;
+			AlignEmbeddedUsingStatements = true;
+			PropertyFormatting = PropertyFormatting.AllowOneLine;
+			BeforeMethodDeclarationParameterComma = false;
+			AfterMethodDeclarationParameterComma = true;
+			BeforeFieldDeclarationComma = false;
+			AfterFieldDeclarationComma = true;
+			BeforeLocalVariableDeclarationComma = false;
+			AfterLocalVariableDeclarationComma = true;
+			
+			BeforeIndexerDeclarationBracket = true;
+			WithinIndexerDeclarationBracket = false;
+			BeforeIndexerDeclarationParameterComma = false;
+		
+		
+			AfterIndexerDeclarationParameterComma = true;
+			
+			BlankLinesBeforeUsings = 0;
+			BlankLinesAfterUsings = 1;
+			
+			
+			BlankLinesBeforeFirstDeclaration = 0;
+			BlankLinesBetweenTypes = 1;
+			BlankLinesBetweenFields = 0;
+			BlankLinesBetweenEventFields = 0;
+			BlankLinesBetweenMembers = 1;
+		}
+		
+		public static CSharpFormattingPolicy Load (FilePath selectedFile)
+		{
+			using (var stream = System.IO.File.OpenRead (selectedFile)) {
+				return Load (stream);
+			}
+		}
+		
+		public static CSharpFormattingPolicy Load (System.IO.Stream input)
+		{
+			CSharpFormattingPolicy result = new CSharpFormattingPolicy ();
+			result.Name = "noname";
+			using (XmlTextReader reader = new XmlTextReader (input)) {
+				while (reader.Read ()) {
+					if (reader.NodeType == XmlNodeType.Element) {
+						if (reader.LocalName == "Property") {
+							var info = typeof (CSharpFormattingPolicy).GetProperty (reader.GetAttribute ("name"));
+							string valString = reader.GetAttribute ("value");
+							object value;
+							if (info.PropertyType == typeof (bool)) {
+								value = Boolean.Parse (valString);
+							} else if (info.PropertyType == typeof (int)) {
+								value = Int32.Parse (valString);
+							} else {
+								value = Enum.Parse (info.PropertyType, valString);
+							}
+							info.SetValue (result, value, null);
+						} else if (reader.LocalName == "FormattingProfile") {
+							result.Name = reader.GetAttribute ("name");
+						}
+					} else if (reader.NodeType == XmlNodeType.EndElement && reader.LocalName == "FormattingProfile") {
+						//Console.WriteLine ("result:" + result.Name);
+						return result;
+					}
+				}
+			}
+			return result;
+		}
+		
+		public void Save (string fileName)
+		{
+			using (var writer = new XmlTextWriter (fileName, Encoding.Default)) {
+				writer.Formatting = System.Xml.Formatting.Indented;
+				writer.Indentation = 1;
+				writer.IndentChar = '\t';
+				writer.WriteStartElement ("FormattingProfile");
+				writer.WriteAttributeString ("name", Name);
+				foreach (PropertyInfo info in typeof (CSharpFormattingPolicy).GetProperties ()) {
+					if (info.GetCustomAttributes (false).Any (o => o.GetType () == typeof(ItemPropertyAttribute))) {
+						writer.WriteStartElement ("Property");
+						writer.WriteAttributeString ("name", info.Name);
+						writer.WriteAttributeString ("value", info.GetValue (this, null).ToString ());
+						writer.WriteEndElement ();
+					}
+				}
+				writer.WriteEndElement ();
+			}
 		}
 		
 		public bool Equals (CSharpFormattingPolicy other)
 		{
 			foreach (PropertyInfo info in typeof (CSharpFormattingPolicy).GetProperties ()) {
-				object val      = info.GetValue (this, null);
-				object otherVal = info.GetValue (other, null);
-				if (!val.Equals (otherVal)) {
-					//Console.WriteLine ("!equal");
-					return false;
+				if (info.GetCustomAttributes (false).Any (o => o.GetType () == typeof(ItemPropertyAttribute))) {
+					object val = info.GetValue (this, null);
+					object otherVal = info.GetValue (other, null);
+					if (val == null) {
+						if (otherVal == null)
+							continue;
+						return false;
+					}
+					if (!val.Equals (otherVal)) {
+						//Console.WriteLine ("!equal");
+						return false;
+					}
 				}
 			}
 			//Console.WriteLine ("== equal");
