@@ -28,19 +28,18 @@
 
 using System;
 using System.Text;
-using System.Text.RegularExpressions;
 using System.Collections;
 using System.Collections.Generic;
-using System.Threading;
 using Gtk;
 using Gdk;
 
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Ide.Gui.Pads;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.NUnit.Commands;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Components.Docking;
+using MonoDevelop.Ide;
+using System.Text.RegularExpressions;
 
 namespace MonoDevelop.NUnit
 {
@@ -51,7 +50,6 @@ namespace MonoDevelop.NUnit
 		IPadWindow window;
 		VBox panel;
 		HPaned book;
-		Gtk.Tooltips tips = new Gtk.Tooltips ();
 		
 		Label infoFailed = new Label (GettextCatalog.GetString ("<b>Failed</b>: {0}", 0));
 		Label infoIgnored = new Label (GettextCatalog.GetString ("<b>Ignored</b>: {0}", 0));
@@ -68,14 +66,15 @@ namespace MonoDevelop.NUnit
 		Dictionary<UnitTest,int> outIters = new Dictionary<UnitTest,int> ();
 		Widget outputViewScrolled;
 		VSeparator infoSep;
+		Gtk.TreeIter startMessageIter;
 		
-		ToolButton buttonStop;
-		ToolButton buttonRun;
+		Button buttonStop;
+		Button buttonRun;
 		
-		ToggleToolButton buttonSuccess;
-		ToggleToolButton buttonFailures;
-		ToggleToolButton buttonIgnored;
-		ToggleToolButton buttonOutput;
+		ToggleButton buttonSuccess;
+		ToggleButton buttonFailures;
+		ToggleButton buttonIgnored;
+		ToggleButton buttonOutput;
 		
 		bool running;
 		int testsToRun;
@@ -103,59 +102,6 @@ namespace MonoDevelop.NUnit
 			
 			panel = new VBox ();
 			
-			Toolbar toolbar = new Toolbar ();
-			toolbar.IconSize = IconSize.Menu;
-			panel.PackStart (toolbar, false, false, 0);
-			
-			buttonSuccess = new ToggleToolButton ();
-			buttonSuccess.Label = GettextCatalog.GetString ("Successful Tests");
-			buttonSuccess.Active = false;
-			buttonSuccess.IconWidget = new Gtk.Image (CircleImage.Success);
-			buttonSuccess.IsImportant = true;
-			buttonSuccess.Toggled += new EventHandler (OnShowSuccessfulToggled);
-			buttonSuccess.SetTooltip (tips, GettextCatalog.GetString ("Show Successful Tests"), GettextCatalog.GetString ("Show Successful Tests"));
-			toolbar.Insert (buttonSuccess, -1);
-			
-			buttonFailures = new ToggleToolButton ();
-			buttonFailures.Label = GettextCatalog.GetString ("Failed Tests");
-			buttonFailures.Active = true;
-			buttonFailures.IconWidget = new Gtk.Image (CircleImage.Failure);
-			buttonFailures.IsImportant = true;
-			buttonFailures.Toggled += new EventHandler (OnShowFailuresToggled);
-			buttonFailures.SetTooltip (tips, GettextCatalog.GetString ("Show Failed Tests"), GettextCatalog.GetString ("Show Failed Tests"));
-			toolbar.Insert (buttonFailures, -1);
-			
-			buttonIgnored = new ToggleToolButton ();
-			buttonIgnored.Label = GettextCatalog.GetString ("Ignored Tests");
-			buttonIgnored.Active = true;
-			buttonIgnored.IconWidget = new Gtk.Image (CircleImage.NotRun);
-			buttonIgnored.Toggled += new EventHandler (OnShowIgnoredToggled);
-			buttonIgnored.IsImportant = true;
-			buttonIgnored.SetTooltip (tips, GettextCatalog.GetString( "Show Ignored Tests"), GettextCatalog.GetString ("Show Ignored Tests"));
-			toolbar.Insert (buttonIgnored, -1);
-			
-			buttonOutput = new ToggleToolButton ();
-			buttonOutput.Label = GettextCatalog.GetString ("Output");
-			buttonOutput.Active = false;
-			buttonOutput.IconWidget = Services.Resources.GetImage (MonoDevelop.Core.Gui.Stock.OutputIcon, IconSize.Menu);
-			buttonOutput.Toggled += new EventHandler (OnShowOutputToggled);
-			buttonOutput.IsImportant = true;
-			buttonOutput.SetTooltip (tips, GettextCatalog.GetString ("Show Output"), GettextCatalog.GetString ("Show Output"));
-			toolbar.Insert (buttonOutput, -1);
-			
-			toolbar.Insert (new SeparatorToolItem (), -1);
-			
-			buttonRun = new ToolButton (new Gtk.Image (Gtk.Stock.Execute, IconSize.Menu), GettextCatalog.GetString ("Run Test"));
-			buttonRun.IsImportant = true;
-			buttonRun.Sensitive = false;
-			toolbar.Insert (buttonRun, -1);
-			
-			buttonStop = new ToolButton (Gtk.Stock.Stop);
-			toolbar.Insert (buttonStop, -1);
-			
-			toolbar.ToolbarStyle = ToolbarStyle.BothHoriz;
-			toolbar.ShowArrow = false;
-			
 			// Results notebook
 			
 			book = new HPaned ();
@@ -165,8 +111,8 @@ namespace MonoDevelop.NUnit
 			// Failures tree
 			failuresTreeView = new TreeView ();
 			failuresTreeView.HeadersVisible = false;
-			failuresStore = new TreeStore (typeof(Pixbuf), typeof (string), typeof(object));
-			CellRendererPixbuf pr = new CellRendererPixbuf ();
+			failuresStore = new TreeStore (typeof(Pixbuf), typeof (string), typeof(object), typeof(string));
+			var pr = new CellRendererPixbuf ();
 			CellRendererText tr = new CellRendererText ();
 			TreeViewColumn col = new TreeViewColumn ();
 			col.PackStart (pr, false);
@@ -176,54 +122,22 @@ namespace MonoDevelop.NUnit
 			failuresTreeView.AppendColumn (col);
 			failuresTreeView.Model = failuresStore;
 		
-			Gtk.ScrolledWindow sw = new Gtk.ScrolledWindow ();
+			var sw = new MonoDevelop.Components.CompactScrolledWindow ();
+			sw.ShadowType = ShadowType.None;
 			sw.Add(failuresTreeView);
-			Frame frame = new Frame ();
-			frame.Add (sw);
-			book.Pack1 (frame, true, true);
+			book.Pack1 (sw, true, true);
 			
 			outputView = new TextView();
 			outputView.Editable = false;
 			bold = new TextTag ("bold");
 			bold.Weight = Pango.Weight.Bold;
 			outputView.Buffer.TagTable.Add (bold);
-			sw = new Gtk.ScrolledWindow ();
+			sw = new MonoDevelop.Components.CompactScrolledWindow ();
+			sw.ShadowType = ShadowType.None;
 			sw.Add(outputView);
-			frame = new Frame ();
-			frame.Add (sw);
-			book.Pack2 (frame, true, true);
-			outputViewScrolled = frame;
+			book.Pack2 (sw, true, true);
+			outputViewScrolled = sw;
 			
-			// Run panel
-			
-			HBox runPanel = new HBox ();
-			runPanel.BorderWidth = 5;
-			
-			infoSep = new VSeparator ();
-			
-			resultLabel.UseMarkup = true;
-			infoCurrent.Ellipsize = Pango.EllipsizeMode.Start;
-			infoCurrent.WidthRequest = 0;
-			runPanel.PackStart (resultLabel, false, false, 0);
-			runPanel.PackStart (progressBar, false, false, 0);
-			runPanel.PackStart (infoCurrent, true, true, 10);	
-			
-			labels = new HBox (false, 10);
-			
-			infoFailed.UseMarkup = true;
-			infoIgnored.UseMarkup = true;
-			
-			labels.PackStart (infoFailed, true, false, 0);
-			labels.PackStart (infoIgnored, true, false, 0);
-			
-			runPanel.PackEnd (labels, false, false, 0);
-			runPanel.PackEnd (infoSep, false, false, 10);
-			
-			panel.PackStart (runPanel, false, false, 0);
-			progressBar.HeightRequest = infoFailed.SizeRequest().Height;
-			
-			buttonStop.Clicked += new EventHandler (OnStopClicked);
-			buttonRun.Clicked += new EventHandler (OnRunClicked);
 			failuresTreeView.ButtonReleaseEvent += new Gtk.ButtonReleaseEventHandler (OnPopupMenu);
 			failuresTreeView.RowActivated += OnRowActivated;
 			failuresTreeView.Selection.Changed += OnRowSelected;
@@ -236,6 +150,88 @@ namespace MonoDevelop.NUnit
 		void IPadContent.Initialize (IPadWindow window)
 		{
 			this.window = window;
+			
+			DockItemToolbar toolbar = window.GetToolbar (PositionType.Top);
+			
+			buttonSuccess = new ToggleButton ();
+			buttonSuccess.Label = GettextCatalog.GetString ("Successful Tests");
+			buttonSuccess.Active = false;
+			buttonSuccess.Image = new Gtk.Image (CircleImage.Success);
+			buttonSuccess.Image.Show ();
+			buttonSuccess.Toggled += new EventHandler (OnShowSuccessfulToggled);
+			buttonSuccess.TooltipText = GettextCatalog.GetString ("Show Successful Tests");
+			toolbar.Add (buttonSuccess);
+			
+			buttonFailures = new ToggleButton ();
+			buttonFailures.Label = GettextCatalog.GetString ("Failed Tests");
+			buttonFailures.Active = true;
+			buttonFailures.Image = new Gtk.Image (CircleImage.Failure);
+			buttonFailures.Image.Show ();
+			buttonFailures.Toggled += new EventHandler (OnShowFailuresToggled);
+			buttonFailures.TooltipText = GettextCatalog.GetString ("Show Failed Tests");
+			toolbar.Add (buttonFailures);
+			
+			buttonIgnored = new ToggleButton ();
+			buttonIgnored.Label = GettextCatalog.GetString ("Ignored Tests");
+			buttonIgnored.Active = true;
+			buttonIgnored.Image = new Gtk.Image (CircleImage.NotRun);
+			buttonIgnored.Image.Show ();
+			buttonIgnored.Toggled += new EventHandler (OnShowIgnoredToggled);
+			buttonIgnored.TooltipText = GettextCatalog.GetString( "Show Ignored Tests");
+			toolbar.Add (buttonIgnored);
+			
+			buttonOutput = new ToggleButton ();
+			buttonOutput.Label = GettextCatalog.GetString ("Output");
+			buttonOutput.Active = false;
+			buttonOutput.Image = ImageService.GetImage (MonoDevelop.Ide.Gui.Stock.OutputIcon, IconSize.Menu);
+			buttonOutput.Image.Show ();
+			buttonOutput.Toggled += new EventHandler (OnShowOutputToggled);
+			buttonOutput.TooltipText = GettextCatalog.GetString ("Show Output");
+			toolbar.Add (buttonOutput);
+			
+			toolbar.Add (new SeparatorToolItem ());
+			
+			buttonRun = new Button ();
+			buttonRun.Label = GettextCatalog.GetString ("Run Test");
+			buttonRun.Image = new Gtk.Image (Gtk.Stock.Execute, IconSize.Menu);
+			buttonRun.Image.Show ();
+			buttonRun.Sensitive = false;
+			toolbar.Add (buttonRun);
+			
+			buttonStop = new Button (new Gtk.Image (Gtk.Stock.Stop, Gtk.IconSize.Menu));
+			toolbar.Add (buttonStop);
+			toolbar.ShowAll ();
+			
+			buttonStop.Clicked += new EventHandler (OnStopClicked);
+			buttonRun.Clicked += new EventHandler (OnRunClicked);
+			
+			// Run panel
+			
+			DockItemToolbar runPanel = window.GetToolbar (PositionType.Bottom);
+			
+			infoSep = new VSeparator ();
+			
+			resultLabel.UseMarkup = true;
+			infoCurrent.Ellipsize = Pango.EllipsizeMode.Start;
+			infoCurrent.WidthRequest = 0;
+			runPanel.Add (resultLabel);
+			runPanel.Add (progressBar);
+			runPanel.Add (infoCurrent, true, 10);	
+			
+			labels = new HBox (false, 10);
+			
+			infoFailed.UseMarkup = true;
+			infoIgnored.UseMarkup = true;
+			
+			labels.PackStart (infoFailed, true, false, 0);
+			labels.PackStart (infoIgnored, true, false, 0);
+			
+			runPanel.Add (new Gtk.Label (), true);
+			runPanel.Add (labels);
+			runPanel.Add (infoSep, false, 10);
+			
+			progressBar.HeightRequest = infoFailed.SizeRequest().Height;
+			runPanel.ShowAll ();
 		}
 		
 		public void Dispose ()
@@ -245,6 +241,10 @@ namespace MonoDevelop.NUnit
 		public void OnTestSuiteChanged (object sender, EventArgs e)
 		{
 			results.Clear ();
+			
+			error = null;
+			errorMessage = null;
+			
 			failuresStore.Clear ();
 			outputView.Buffer.Clear ();
 			outIters.Clear ();
@@ -265,11 +265,7 @@ namespace MonoDevelop.NUnit
 			get { return running; }
 			set {
 				running = value;
-				string title = GettextCatalog.GetString ("Test results");
-				if (running) 
-					window.Title = "<span foreground=\"blue\">" + title + "</span>";
-				else
-					window.Title = title;
+				window.IsWorking = value;
 			}
 		}
 		
@@ -294,6 +290,10 @@ namespace MonoDevelop.NUnit
 			rootTest = test;
 			results.Clear ();
 			testsToRun = test.CountTestCases ();
+			
+			error = null;
+			errorMessage = null;
+			
 			progressBar.Fraction = 0;
 			progressBar.Text = "";
 			progressBar.Text = "0 / " + testsToRun;
@@ -317,17 +317,19 @@ namespace MonoDevelop.NUnit
 			cancel = false;
 			Running = true;
 			
-			configuration = IdeApp.Workspace.ActiveConfiguration;
+			configuration = IdeApp.Workspace.ActiveConfigurationId;
 			
 			AddStartMessage ();
 		}
 		
-		public void AddStartMessage ()
+		public void AddStartMessage (bool isRunning = true)
 		{
 			if (rootTest != null) {
 				Gdk.Pixbuf infoIcon = failuresTreeView.RenderIcon (Gtk.Stock.DialogInfo, Gtk.IconSize.Menu, "");
-				string msg = string.Format (GettextCatalog.GetString ("Running tests for <b>{0}</b> configuration <b>{1}</b>"), rootTest.Name, configuration);
-				failuresStore.AppendValues (infoIcon, msg, rootTest);
+				string msg = string.Format (isRunning ? GettextCatalog.GetString ("Running tests for <b>{0}</b> configuration <b>{1}</b>") : GettextCatalog.GetString ("Test results for <b>{0}</b> configuration <b>{1}</b>"), rootTest.Name, configuration);
+				startMessageIter = failuresStore.AppendValues (infoIcon, msg, rootTest);
+			} else {
+				startMessageIter = Gtk.TreeIter.Zero;
 			}
 		}
 
@@ -348,11 +350,31 @@ namespace MonoDevelop.NUnit
 			TreeIter testRow = failuresStore.AppendValues (stock, msg, null);
 			failuresStore.AppendValues (testRow, null, Escape (error.GetType().Name + ": " + error.Message), null);
 			TreeIter row = failuresStore.AppendValues (testRow, null, GettextCatalog.GetString ("Stack Trace"), null);
-			failuresStore.AppendValues (row, null, Escape (error.StackTrace), null);
+			AddStackTrace (row, error.StackTrace, null);
+		}
+		
+		void AddStackTrace (TreeIter row, string stackTrace, UnitTest test)
+		{
+			string[] stackLines = stackTrace.Replace ("\r","").Split ('\n');
+			foreach (string line in stackLines) {
+				Regex r = new Regex (@".*?\(.*?\)\s\[.*?\]\s.*?\s(?<file>.*)\:(?<line>\d*)");
+				Match m = r.Match (line);
+				string file;
+				if (m.Groups["file"] != null && m.Groups["line"] != null)
+					file = m.Groups["file"].Value + ":" + m.Groups["line"].Value;
+				else
+					file = null;
+				failuresStore.AppendValues (row, null, Escape (line), test, file);
+			}
 		}
 		
 		public void FinishTestRun ()
 		{
+			if (!Gtk.TreeIter.Zero.Equals (startMessageIter)) {
+				string msg = string.Format (GettextCatalog.GetString ("Test results for <b>{0}</b> configuration <b>{1}</b>"), rootTest.Name, configuration);
+				failuresStore.SetValue (startMessageIter, 1, msg);
+				startMessageIter = Gtk.TreeIter.Zero;
+			}
 			infoCurrent.Text = "";
 			progressBar.Fraction = 1;
 			progressBar.Text = "";
@@ -396,6 +418,20 @@ namespace MonoDevelop.NUnit
 
 		void OnRowActivated (object s, EventArgs a)
 		{
+			Gtk.TreeIter iter;
+			if (failuresTreeView.Selection.GetSelected (out iter)) {
+				string file = (string) failuresStore.GetValue (iter, 3);
+				if (file != null) {
+					int i = file.LastIndexOf (':');
+					if (i != -1) {
+						int line;
+						if (int.TryParse (file.Substring (i+1), out line)) {
+							IdeApp.Workbench.OpenDocument (file.Substring (0, i), line, -1);
+							return;
+						}
+					}
+				}
+			}
 			OnShowTest ();
 		}
 		
@@ -429,7 +465,7 @@ namespace MonoDevelop.NUnit
 			info.Enabled = test != null;
 		}
 		
-		[CommandHandler (TestCommands.ShowTestCode)]
+		[CommandHandler (TestCommands.GoToFailure)]
 		protected void OnShowTest ()
 		{
 			UnitTest test = GetSelectedTest ();
@@ -442,10 +478,22 @@ namespace MonoDevelop.NUnit
 			if (loc == null)
 				loc = test.SourceCodeLocation;
 			if (loc != null)
-				IdeApp.Workbench.OpenDocument (loc.FileName, loc.Line, loc.Column, true);
+				IdeApp.Workbench.OpenDocument (loc.FileName, loc.Line, loc.Column);
+		}
+		
+		[CommandHandler (TestCommands.ShowTestCode)]
+		protected void OnShowTestCode ()
+		{
+			UnitTest test = GetSelectedTest ();
+			if (test == null)
+				return;
+			SourceCodeLocation loc = test.SourceCodeLocation;
+			if (loc != null)
+				IdeApp.Workbench.OpenDocument (loc.FileName, loc.Line, loc.Column);
 		}
 		
 		[CommandUpdateHandler (TestCommands.ShowTestCode)]
+		[CommandUpdateHandler (TestCommands.GoToFailure)]
 		protected void OnUpdateRunTest (CommandInfo info)
 		{
 			UnitTest test = GetSelectedTest ();
@@ -488,7 +536,7 @@ namespace MonoDevelop.NUnit
 			failuresStore.Clear ();
 			outputView.Buffer.Clear ();
 			outIters.Clear ();
-			AddStartMessage ();
+			AddStartMessage (running);
 				
 			foreach (ResultRecord res in results) {
 				ShowTestResult (res.Test, res.Result);
@@ -509,7 +557,8 @@ namespace MonoDevelop.NUnit
 			if (result.IsFailure) {
 				if (!buttonFailures.Active)
 					return;
-				TreeIter testRow = failuresStore.AppendValues (CircleImage.Failure, Escape (test.FullName), test);
+				string file = test.SourceCodeLocation != null ? test.SourceCodeLocation.FileName + ":" + test.SourceCodeLocation.Line : null;
+				TreeIter testRow = failuresStore.AppendValues (CircleImage.Failure, Escape (test.FullName), test, file);
 				bool hasMessage = result.Message != null && result.Message.Length > 0;
 				if (hasMessage)
 					failuresStore.AppendValues (testRow, null, Escape (result.Message), test);
@@ -517,7 +566,7 @@ namespace MonoDevelop.NUnit
 					TreeIter row = testRow;
 					if (hasMessage)
 						row = failuresStore.AppendValues (testRow, null, GettextCatalog.GetString ("Stack Trace"), test);
-					failuresStore.AppendValues (row, null, Escape (result.StackTrace), test);
+					AddStackTrace (row, result.StackTrace, test);
 				}
 				failuresTreeView.ScrollToCell (failuresStore.GetPath (testRow), null, false, 0, 0);
 			}
