@@ -31,6 +31,7 @@ using System.Collections.Generic;
 using System.Collections.ObjectModel;
 using System.Text;
 using System.Xml;
+using System.Linq;
 using MonoDevelop.Projects.Dom.Parser;
 using MonoDevelop.Core;
 
@@ -130,7 +131,7 @@ namespace MonoDevelop.Projects.Dom
 					result.Append ("M:");
 					result.Append (FullName);
 					if (TypeParameters.Count > 0) {
-						result.Append ("~");
+						result.Append ("`");
 						result.Append (TypeParameters.Count);
 					}
 				}
@@ -172,78 +173,145 @@ namespace MonoDevelop.Projects.Dom
 			this.MethodModifier = MethodModifier;
 		}
 		
+		class ReturnTypeStepper
+		{
+			Stack<IReturnType> stack = new Stack<IReturnType> ();
+			
+			public IReturnType Cur {
+				get {
+					return stack.Peek ();
+				}
+			}
+			
+			public bool HasNext {
+				get {
+					return stack.Count > 0;
+				}
+			}
+			
+			public ReturnTypeStepper (IReturnType returnType)
+			{
+				stack .Push (returnType);
+			}
+			
+			public bool Step ()
+			{
+				if (stack.Count == 0)
+					return false;
+				IReturnType cur = stack.Pop ();
+				foreach (var generic in cur.GenericArguments.Reverse ()) {
+					stack.Push (generic);
+				}
+				return true;
+			}
+			
+			public bool Skip ()
+			{
+				if (stack.Count == 0)
+					return false;
+				stack.Pop ();
+				return true;
+			}
+			
+			
+		}
+		
 		public static IMethod CreateInstantiatedGenericMethod (IMethod method, IList<IReturnType> genericArguments, IList<IReturnType> methodArguments)
 		{
-//			System.Console.WriteLine("----");
-//			Console.WriteLine ("instantiate: " + method);
+			//System.Console.WriteLine("----");
+			//Console.WriteLine ("instantiate: " + method);
 			GenericMethodInstanceResolver resolver = new GenericMethodInstanceResolver ();
 			if (genericArguments != null) {
 				for (int i = 0; i < method.TypeParameters.Count && i < genericArguments.Count; i++) 
-					resolver.Add (new DomReturnType (method.TypeParameters[i].Name), genericArguments[i]);
+					resolver.Add (method.DeclaringType != null ? method.DeclaringType.SourceProjectDom : null, new DomReturnType (method.TypeParameters [i].Name), genericArguments [i]);
 			}
 			IMethod result = (IMethod)method.AcceptVisitor (resolver, method);
 			resolver = new GenericMethodInstanceResolver ();
-			if (methodArguments != null) {
-				// The stack should contain <TEMPLATE> / RealType pairs
-				Stack<KeyValuePair<IReturnType, IReturnType>> returnTypeStack = new Stack<KeyValuePair<IReturnType, IReturnType>> ();
-				for (int i = 0; i < method.Parameters.Count && i < methodArguments.Count; i++) {
-//					Console.WriteLine ("parameter:" + method.Parameters[i]);
-					returnTypeStack.Push (new KeyValuePair<IReturnType, IReturnType> (method.Parameters[i].ReturnType, methodArguments[i]));
-					while (returnTypeStack.Count > 0) {
-						KeyValuePair<IReturnType, IReturnType> curReturnType = returnTypeStack.Pop ();
-//						Console.WriteLine ("key:" + curReturnType.Key + "/ val:" + curReturnType.Value);
-						bool found = false;
-						for (int j = 0; j < method.TypeParameters.Count; j++) {
-							if (method.TypeParameters[j].Name == curReturnType.Key.FullName) {
-								found = true;
-								break;
-							}
-						}
-						if (found) {
-							resolver.Add (curReturnType.Key, curReturnType.Value);
-							continue;
-						}
-						//Console.WriteLine ("key:" + curReturnType.Key);
-						//Console.WriteLine ("value:" + curReturnType.Value);
-						for (int k = 0; k < System.Math.Min (curReturnType.Key.GenericArguments.Count, curReturnType.Value.GenericArguments.Count); k++) {
-							//Console.WriteLine ("add " + curReturnType.Key.GenericArguments[k] + " " + curReturnType.Value.GenericArguments[k]);
-							returnTypeStack.Push (new KeyValuePair<IReturnType, IReturnType> (curReturnType.Key.GenericArguments[k], 
-							                                                                  curReturnType.Value.GenericArguments[k]));
-						}
+			var containerSet = new HashSet<ITypeParameter> (method.TypeParameters);
+			
+			for (int i = 0; i < method.Parameters.Count && i < methodArguments.Count; i++) {
+				ReturnTypeStepper original = new ReturnTypeStepper (method.Parameters [i].ReturnType);
+				ReturnTypeStepper argument = new ReturnTypeStepper (methodArguments [i]);
+				while (original.HasNext && argument.HasNext) {
+					if (containerSet.Any (p => p.Name == original.Cur.DecoratedFullName)) {
+						resolver.Add (method.DeclaringType != null ? method.DeclaringType.SourceProjectDom : null, original.Cur, argument.Cur);
+						original.Skip ();
+						argument.Skip ();
+					} else {
+						original.Step ();
+						argument.Step ();
 					}
 				}
 			}
-//			System.Console.WriteLine("before:" + result);
+			
+			//			if (methodArguments != null) {
+			//				// The stack should contain <TEMPLATE> / RealType pairs
+			//				Stack<KeyValuePair<IReturnType, IReturnType>> returnTypeStack = new Stack<KeyValuePair<IReturnType, IReturnType>> ();
+			//				for (int i = 0; i < method.Parameters.Count && i < methodArguments.Count; i++) {
+			//					returnTypeStack.Push (new KeyValuePair<IReturnType, IReturnType> (method.Parameters[i].ReturnType, methodArguments[i]));
+			//					while (returnTypeStack.Count > 0) {
+			//						KeyValuePair<IReturnType, IReturnType> curReturnType = returnTypeStack.Pop ();
+			//						//Console.WriteLine ("key:" + curReturnType.Key + "\n val:" + curReturnType.Value);
+			//						bool found = false;
+			//						for (int j = 0; j < method.TypeParameters.Count; j++) {
+			//							if (method.TypeParameters[j].Name == curReturnType.Key.FullName) {
+			//								found = true;
+			//								break;
+			//							}
+			//						}
+			//						if (found) {
+			//							resolver.Add (method.DeclaringType != null ? method.DeclaringType.SourceProjectDom : null, curReturnType.Key, curReturnType.Value);
+			//							continue;
+			//						}
+			//						//Console.WriteLine ("key:" + curReturnType.Key);
+			//						//Console.WriteLine ("value:" + curReturnType.Value);
+			//						for (int k = 0; k < System.Math.Min (curReturnType.Key.GenericArguments.Count, curReturnType.Value.GenericArguments.Count); k++) {
+			//							//Console.WriteLine ("add " + curReturnType.Key.GenericArguments[k] + " " + curReturnType.Value.GenericArguments[k]);
+			//							returnTypeStack.Push (new KeyValuePair<IReturnType, IReturnType> (curReturnType.Key.GenericArguments[k], 
+			//							                                                                  curReturnType.Value.GenericArguments[k]));
+			//						}
+			//					}
+			//				}
+			//			}
+			//			System.Console.WriteLine("before:" + result);
 			result = (IMethod)result.AcceptVisitor (resolver, result);
 			((DomMethod)result).DeclaringType = method.DeclaringType;
 			((DomMethod)result).MethodModifier = method.MethodModifier;
 			
-//			System.Console.WriteLine("after:" + result);
-//			Console.WriteLine (result.Parameters[0]);
+			//			System.Console.WriteLine("after:" + result);
+			//			Console.WriteLine (result.Parameters[0]);
 			return result;
 		}
 		
-		internal class GenericMethodInstanceResolver: CopyDomVisitor<IMethod>
+		public class GenericMethodInstanceResolver: CopyDomVisitor<IMethod>
 		{
 			public Dictionary<string, IReturnType> typeTable = new Dictionary<string,IReturnType> ();
 			
-			public void Add (IReturnType parameterType, IReturnType type)
+			public void Add (ProjectDom dom, IReturnType parameterType, IReturnType type)
 			{
-				//Console.WriteLine ("Add:" + parameterType +"->" + type);
+//				Console.WriteLine ("Add:" + parameterType +"\n\t->" + type);
 				if (type == null || string.IsNullOrEmpty (type.FullName))
 					return;
 				string name = parameterType.Name;
-				if (!typeTable.ContainsKey (name)) {
-					DomReturnType newType = new DomReturnType (type.FullName);
-					newType.ArrayDimensions     = Math.Max (0, type.ArrayDimensions - parameterType.ArrayDimensions);
-					newType.PointerNestingLevel = Math.Max (0, type.PointerNestingLevel - parameterType.PointerNestingLevel);
-					newType.Type  = type.Type; // May be anonymous type
-					for (int i = 0; i < newType.ArrayDimensions; i++)
-						newType.SetDimension (i, parameterType.GetDimension (i));
-					foreach (var generic in type.GenericArguments)
-						newType.AddTypeParameter (generic);
-					typeTable[name] = newType;
+				bool contains = typeTable.ContainsKey (name);
+				
+				// when the type is already in the table use the type that is more general in the inheritance tree.
+				if (contains && dom != null) {
+					var t1 = dom.GetType (typeTable[name]);
+					var t2 = dom.GetType (type);
+					if (t1 != null && !dom.GetInheritanceTree (t1).Any (t => t.DecoratedFullName == t2.DecoratedFullName))
+						return;
 				}
+				
+				DomReturnType newType = new DomReturnType (type.FullName);
+				newType.ArrayDimensions     = Math.Max (0, type.ArrayDimensions - parameterType.ArrayDimensions);
+				newType.PointerNestingLevel = Math.Max (0, type.PointerNestingLevel - parameterType.PointerNestingLevel);
+				newType.Type  = type.Type; // May be anonymous type
+				for (int i = 0; i < newType.ArrayDimensions; i++)
+					newType.SetDimension (i, parameterType.GetDimension (i));
+				foreach (var generic in type.GenericArguments)
+					newType.AddTypeParameter (generic);
+				typeTable[name] = newType;
 			}
 			
 			public override INode Visit (IMethod source, IMethod data)
@@ -274,7 +342,8 @@ namespace MonoDevelop.Projects.Dom
 						DomReturnType drr = new DomReturnType (res.FullName);
 						drr.PointerNestingLevel = type.PointerNestingLevel;
 						drr.ArrayDimensions = type.ArrayDimensions;
-						drr.Type  = type.Type; // May be anonymous type
+						if (!(type.Type is ITypeParameterType))
+							drr.Type  = type.Type; // May be anonymous type
 						for (int i = 0; i < type.ArrayDimensions; i++)
 							drr.SetDimension (i, type.GetDimension (i));
 						return drr;
@@ -298,9 +367,9 @@ namespace MonoDevelop.Projects.Dom
 			if (dom == null || type == null || Parameters.Count == 0 || !IsExtension) {
 				return null;
 			}
-//			Console.WriteLine ("Ext.Type: " + type);
 			string extensionTableKey = this.HelpUrl + "/" + type.FullName;
-//			Console.WriteLine ("table key:" + extensionTableKey);
+			if (Name.StartsWith ("ForEachWithIndex"))
+				Console.WriteLine ("table key:" + extensionTableKey);
 			lock (extensionTable) {
 				if (extensionTable.ContainsKey (extensionTableKey))
 					return extensionTable[extensionTableKey];
@@ -324,14 +393,15 @@ namespace MonoDevelop.Projects.Dom
 					string baseTypeFullName = baseType is InstantiatedType ? ((InstantiatedType)baseType).UninstantiatedType.FullName : baseType.FullName;
 					
 					// compare the generic arguments.
-					if (instMethod.Parameters[0].ReturnType.FullName == baseTypeFullName) {
+					if (instMethod.Parameters[0].ReturnType.FullName == baseTypeFullName && Parameters[0].ReturnType.ArrayDimensions == 0 && Parameters[0].ReturnType.PointerNestingLevel == 0) {
 						if (instMethod.Parameters[0].ReturnType.GenericArguments.Count > 0) {
 							InstantiatedType instType = baseType as InstantiatedType;
 							if (instType == null || instType.GenericParameters.Count != instMethod.Parameters[0].ReturnType.GenericArguments.Count)
 								continue;
 							bool genericArgumentsAreEqual = true;
 							for (int i = 0; i < instMethod.Parameters[0].ReturnType.GenericArguments.Count; i++) {
-								//Console.WriteLine (instMethod.Parameters[0].ReturnType.GenericArguments[i].DecoratedFullName + " --- " + instType.GenericParameters[i].DecoratedFullName);
+								if (Name.StartsWith ("ForEachWithIndex"))
+									Console.WriteLine (instMethod.Parameters[0].ReturnType.GenericArguments[i].DecoratedFullName + " --- " + instType.GenericParameters[i].DecoratedFullName);
 								if (instMethod.Parameters[0].ReturnType.GenericArguments[i].DecoratedFullName != instType.GenericParameters[i].DecoratedFullName) {
 									genericArgumentsAreEqual = false;
 									break;
@@ -354,7 +424,7 @@ namespace MonoDevelop.Projects.Dom
 			}
 		}
 		
-		public void Add (IParameter parameter)
+		public override void Add (IParameter parameter)
 		{
 			if (parameters == null) 
 				parameters = new List<IParameter> ();
@@ -362,15 +432,6 @@ namespace MonoDevelop.Projects.Dom
 			parameters.Add (parameter);
 		}
 		
-		public void Add (IEnumerable<IParameter> parameters)
-		{
-			if (parameters == null)
-				return;
-			foreach (IParameter parameter in parameters) {
-				Add (parameter);
-			}
-		}
-		
 		XmlNode FindMatch (XmlNodeList nodes)
 		{
 			List<IParameter> p = parameters ?? new List<IParameter> ();
