@@ -28,6 +28,7 @@
 using System;
 using System.Collections.Generic;
 using MonoDevelop.Core;
+using System.Collections;
 
 namespace MonoDevelop.Ide.Gui.Components
 {
@@ -406,11 +407,16 @@ namespace MonoDevelop.Ide.Gui.Components
 			
 			bool IsChildIter (Gtk.TreeIter pit, Gtk.TreeIter cit, bool recursive)
 			{
-				while (tree.Store.IterParent (out cit, cit)) {
-					if (cit.Equals (pit)) return true;
-					if (!recursive) return false;
-				}
-				return false;
+				Gtk.TreePath pitPath = tree.Store.GetPath (pit);
+				Gtk.TreePath citPath = tree.Store.GetPath (cit);
+
+				if (!citPath.Up ())
+					return false;
+
+				if (citPath == pitPath)
+					return true;
+
+				return recursive && pitPath.IsAncestor (citPath);
 			}
 	
 			public ITreeNavigator Clone ()
@@ -522,6 +528,14 @@ namespace MonoDevelop.Ide.Gui.Components
 			{
 				AddChild (dataObject, false);
 			}
+			
+			public void AddChildren (IEnumerable dataObjects)
+			{
+				foreach (object dataObject in dataObjects) {
+					AddChild (dataObject, false);
+				}
+			}
+
 	
 			public void AddChild (object dataObject, bool moveToChild)
 			{
@@ -671,6 +685,8 @@ namespace MonoDevelop.Ide.Gui.Components
 			public bool Deleted;
 			public bool Modified;
 			public bool Reset;
+			public bool DeleteDone;
+			public bool ChildrenDeleted;
 
 			public bool HasIter {
 				get { return !NodeIter.Equals (Gtk.TreeIter.Zero); }
@@ -679,13 +695,14 @@ namespace MonoDevelop.Ide.Gui.Components
 
 		internal class TransactedNodeStore
 		{
-			Dictionary<Gtk.TreeIter, TreeNode> iterToNode = new Dictionary<Gtk.TreeIter,TreeNode> ();
+			Dictionary<Gtk.TreeIter, TreeNode> iterToNode;
 			Dictionary<object, List<TreeNode>> objects = new Dictionary<object,List<TreeNode>> ();
 			ExtensibleTreeView tree;
 			
 			public TransactedNodeStore (ExtensibleTreeView tree)
 			{
 				this.tree = tree;
+				iterToNode = new Dictionary<Gtk.TreeIter,TreeNode> (new IterComparer (tree.store));
 			}
 			
 			public TreeNode GetNode (Gtk.TreeIter it)
@@ -712,6 +729,16 @@ namespace MonoDevelop.Ide.Gui.Components
 					if (!objects.TryGetValue (node.DataItem, out nodes)) {
 						nodes = new List<TreeNode> ();
 						objects [node.DataItem] = nodes;
+						
+						// We can't wait for the node commit to fire the added event. A node builder may subscribe
+						// events in OnNodeAdded which may be fired during the transaction.
+						foreach (NodeBuilder nb in node.BuilderChain) {
+							try {
+								nb.OnNodeAdded (node.DataItem);
+							} catch (Exception ex) {
+								LoggingService.LogError (ex.ToString ());
+							}
+						}
 					}
 					nodes.Add (node);
 				}
@@ -727,8 +754,16 @@ namespace MonoDevelop.Ide.Gui.Components
 				if (!node.HasIter) {
 					List<TreeNode> list = objects [node.DataItem];
 					list.Remove (node);
-					if (list.Count == 0)
+					if (list.Count == 0) {
 						objects.Remove (node.DataItem);
+						foreach (NodeBuilder nb in node.BuilderChain) {
+							try {
+								nb.OnNodeRemoved (node.DataItem);
+							} catch (Exception ex) {
+								LoggingService.LogError (ex.ToString ());
+							}
+						}
+					}
 				}
 			}
 
@@ -766,30 +801,48 @@ namespace MonoDevelop.Ide.Gui.Components
 
 			public void CommitChanges ()
 			{
-				foreach (TreeNode node in iterToNode.Values)
+				// First of all, mark as deleted the children of deleted nodes
+				// This avoids trying to delete nodes that have already been deleted (it sets the DeletedDone flag)
+				foreach (TreeNode node in iterToNode.Values) {
+					if (node.Deleted || (node.Reset && !node.Filled))
+						MarkChildrenDeleted (node);
+				}
+				foreach (TreeNode node in iterToNode.Values) {
 					CommitNode (node);
+				}
 			}
 
 			void CommitNode (TreeNode node)
 			{
 				if (node.Deleted) {
+					if (node.DeleteDone)
+						// It means that a parent node has been deleted, so we can skip this one
+						return;
+					
+					MarkDeleted (node);
 					Gtk.TreeIter it = node.NodeIter;
-					tree.UnregisterNode (node.DataItem, it, node.BuilderChain);
 					if (tree.Store.IterIsValid (it))
 						tree.Store.Remove (ref it);
 					return;
 				}
 				else if (node.Modified) {
+					if (!this.tree.store.IterIsValid (node.NodeIter)) {
+						LoggingService.LogError ("Found invalid iter for node. " + node.DataItem);
+						return;
+					}
 					tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.TextColumn, node.Text);
-					tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.OpenIconColumn, node.Icon);
-					tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.ClosedIconColumn, node.ClosedIcon);
+					if (node.Icon != null)
+						tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.OpenIconColumn, node.Icon);
+					if (node.ClosedIcon != null)
+						tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.ClosedIconColumn, node.ClosedIcon);
 				}
 				if (node.Children != null) {
 					foreach (TreeNode cn in node.Children) {
 						Gtk.TreeIter it = tree.Store.AppendValues (node.NodeIter, cn.Text, cn.Icon, cn.ClosedIcon, cn.DataItem, cn.BuilderChain, cn.Filled);
 						if (!cn.Filled)
 							tree.Store.AppendNode (it);	// Dummy node
-						tree.RegisterNode (it, cn.DataItem, cn.BuilderChain);
+						// The OnNodeAdded event was already fired when the node was added. There is no need to fire it again.
+						tree.RegisterNode (it, cn.DataItem, cn.BuilderChain, false);
 						cn.NodeIter = it;
 						cn.Modified = false;
 						node.Reset = false;
@@ -808,6 +861,51 @@ namespace MonoDevelop.Ide.Gui.Components
 				if (node.Expanded)
 					tree.Tree.ExpandToPath (tree.Store.GetPath (node.NodeIter));
 			}
+			
+			void MarkDeleted (TreeNode node)
+			{
+				// Marks the node as deleted and unregisters it from the store
+				// It also marks is children as deleted
+				if (node.DeleteDone)
+					return;
+				node.DeleteDone = true;
+				tree.UnregisterNode (node.DataItem, node.NodeIter, node.BuilderChain, true);
+				MarkChildrenDeleted (node);
+			}
+			
+			void MarkChildrenDeleted (TreeNode node)
+			{
+				// Marks all the children of the node as deleted, and unregisters them from the store
+				if (node.ChildrenDeleted)
+					return;
+				node.ChildrenDeleted = true;
+				if (node.Children != null) {
+					foreach (TreeNode cn in node.Children)
+						MarkDeleted (cn);
+				}
+				// It may have children not instantiated as TreeNode
+				if (!node.NodeIter.Equals (Gtk.TreeIter.Zero))
+					MarkChildrenDeleted (node.NodeIter);
+			}
+			
+			void MarkChildrenDeleted (Gtk.TreeIter it)
+			{
+				Gtk.TreeIter child;
+				if (!tree.store.IterChildren (out child, it))
+					return;
+				do {
+					TreeNode node;
+					if (iterToNode.TryGetValue (child, out node)) {
+						MarkDeleted (node);
+					} else {
+						MarkChildrenDeleted (child);
+						object childData = tree.store.GetValue (child, ExtensibleTreeView.DataItemColumn);
+						if (childData != null)
+							tree.UnregisterNode (childData, child, null, true);
+					}
+				}
+				while (tree.store.IterNext (ref child));
+			}
 		}
 	}
 }
